% CROSS-CORRELATION ANALYSIS
%
% This script analyzes the cross-correlation of two signals. It was developed 
% to identiy asymmetric signal disctributions in microscopic images. The
% output of the script compares the cross-correlation of two images upon
% lateral shift (top left panel). In addition, it also calculates the
% statistical significance of the result using two randomization controls.
% In detail, we use randomization of one signal input (top right panel) and
% randomization of the output (bottom left) to assess the signal distribution. 
% From this, we calculate the sigma. We defined 4 sigma (p < 0.01%) as
% significance threshold. This threshold are depicted in the averages shown
% in the bottom right panel.
%
%
% NOTE: We would like to stress that background substraction as well as contrast 
% enhancement significantly alter the output of the cross-correlation
% analysis and should be avoided.
%
%
% If used, please cite: 
% Matis et al, 2012, Developmental Dynamics, PMID: 22689329.
% A universal analysis tool for the detection of asymmetric signal distribution 
% in microscopic images
%
%
% DISCLOSURE: Wo do not take any responsibility for the analysis and interpretation 
% of experiments using this analysis method.  
%
%
% Copyright. Maja Matis, Jeff Axelrod and Milos Galic. Stanford, 2012


%% IMAGES & PARAMETERS

% NOTE: Please change HERE parameters according to needs

clear all

picture1 =imread('control_center.tif');         % reference signal (for simplicity, it can be assumed this one is the chanel that is not moved)
picture2 =imread('control_20-left.tif');          % polarized signal (interpretation of signal: how many pixels does it need to be moved for maximal ovelap)

p1 = 25;                                        % start of analysis (distance from sides within image)
shift = 50                                      % lateral shift in pixel (total number shown in cross-correlation)
rot = 00                                        % rotation of image in degree. Positive is clockwise. (ATTENTION: Note that rectangular image shape may affect analysis. To avoid this, circular images may be used)

%% ANALYSIS 

% image height & width 
px = size(picture2,1);     
py = size(picture2,2);      

% randomize polarized image: individual columns were randomized in horizontal direction
picture3=picture2;
for rows=1:size(picture2,1)
    vec1=picture2(rows,:);
    length_vec1=size(vec1,2);
    ran_vec1=rand(1,length_vec1);
    [B,ind_vec1]=sort(ran_vec1,'ascend');
    new_vec1=vec1(ind_vec1);
    picture3(rows,:)=new_vec1;
end

% turn images vertical for analysis
picture1=imrotate(picture1,(-90+rot));
picture2=imrotate(picture2,(-90+rot));
picture3=imrotate(picture3,-90);
       

% heatmap: cross-correlation polarized image vs. reference image (X-corr is vertical)
c=[];                                   
for ll=p1-1:(px-p1)                        
x = single(picture1(p1:(py-p1),ll,1));      
pic_1=x-mean(x);
y = single(picture2(p1:(py-p1),ll,1));
pic_2=y-mean(y);
c =[c, xcorr(pic_1,pic_2,shift, 'coeff')];
% c = scatter(x,y) 
end
c=imrotate(c,90);           % return horizontal

% heatmap: cross-correlation randomized image vs. reference image
c_rand=[];                              
for ll=p1-1:(px-p1)                         
x = single(picture1(p1:(py-p1),ll,1));   
pic_1=x-mean(x);
y = single(picture3(p1:(py-p1),ll,1));
pic_3=y-mean(y);
c_rand =[c_rand, xcorr(pic_1,pic_3,shift, 'coeff')];
% c = scatter(x,y) 
end
c_rand=imrotate(c_rand,90);  % return horizontal

% randomize heatmap (polarized image vs reference image): individual rows were randomized in horizontal direction
hm_rand=c;
for rows=1:size(hm_rand,1)
    vec1=hm_rand(rows,:);
    length_vec1=size(vec1,2);
    ran_vec1=rand(1,length_vec1);
    [B,ind_vec1]=sort(ran_vec1,'ascend');
    new_vec1=vec1(ind_vec1);
    hm_rand(rows,:)=new_vec1;
end


%% VALUES

% optional contols
Mean_c=mean(c(:));
Mean_hm=mean(hm_rand(:));
Mean_crand=mean(c_rand(:));

Sigma_c=Std(c(:));
Sigma_rand_heatmap=Std(hm_rand(:));
Sigma_rand_image=Std(c_rand(:));

% p-value (p<0.0001) 
Std_c_col=mean(c,1);
Std_crand_col=mean(c_rand,1); 
Stdev=Std(Std_crand_col);   
Stdev=(4*Stdev);
Std_hm=mean(hm_rand,1);
Stdev1=Std(Std_hm);         
Stdev1=(4*Stdev1);


%% FIGURE

dist1=Mean_crand+Stdev;
dist2=Mean_crand;
dist3=Mean_hm+Stdev1;
dist4=Mean_hm;

% plot1: heatmap (true)
subplot(2,2,1),imagesc(c, [-0.2 0.5]);colorbar('FontSize', 12); 
axis([0 2*shift 0 (px-(2*p1))]);            
grid on;
title('Fig1: X-corr','FontSize',16);
xlabel('lateral dislocation (in pixel)','FontSize',12);
ylabel('Individual rows (y-axis)','FontSize',12);

% plot2: heatmap (scrambled image)
subplot(2,2,2),imagesc(c_rand, [-0.2 0.5]);colorbar('FontSize', 12); 
axis([0 2*shift 0 (px-(2*p1))]);
grid on;
title('Fig2: image (randomized)','FontSize',16);
xlabel('lateral dislocation (in pixel)','FontSize',12);
ylabel('Individual rows (y-axis)','FontSize',12);

% plot3: heatmap (scrambled heatmap)
subplot(2,2,3),imagesc(hm_rand, [-0.2 0.5]);colorbar('FontSize', 12); 
axis([0 2*shift 0 (px-(2*p1))]);            
grid on;
title('Fig3: heatmap (randomized)','FontSize',16);
xlabel('lateral dislocation (in pixel)','FontSize',12);
ylabel('Individual rows (y-axis)','FontSize',12);

% plot4: average cross-correlation
subplot(2,2,4);plot(-shift:shift, mean(c,1),'Color','red','LineWidth',3), hold on; 
plot(-shift:shift, mean(c_rand,1),'Color','green','LineWidth',2),hold on;
plot(-shift:shift, mean(hm_rand,1),'Color','blue','LineWidth',2),hold on;
line([-shift ; +shift],[dist1 ; dist1],'Color','green','LineWidth',2),hold on;
line([-shift ; +shift],[dist2 ; dist2],'Color','green','LineWidth',1),hold on;
line([-shift ; +shift],[dist3 ; dist3],'Color','blue','LineWidth',2),hold on;
line([-shift ; +shift],[dist4 ; dist4],'Color','blue','LineWidth',1),hold on;
axis([-shift shift -0.3 0.7]);      
grid on;
title('Crosscorrelation Analysis','FontSize',16);
xlabel('lateral dislocation (in pixel)','FontSize',12);
ylabel('Score','FontSize',12);
legend('Fig1: mean value','Fig.2: mean & p<0.0001','Fig.3: mean & p<0.0001');
hold on

%% DATA

Fig1_red_max_amplitude=max(Std_c_col)
Fig2_green_mean=Mean_crand
Fig2_green_p_value=Stdev
Fig3_blue_mean=Mean_hm
Fig3_blue_p_value=Stdev1
Ratio_red_green=(Fig1_red_max_amplitude/Fig2_green_p_value)
Ratio_red_blue=(Fig1_red_max_amplitude/Fig3_blue_p_value)

